// LifeView.cpp : implementation of the CLifeView class
//

#include "stdafx.h"
#include "Life.h"

#include "LifeDoc.h"
#include "LifeView.h"
#include "ControlDlg.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CLifeView

IMPLEMENT_DYNCREATE(CLifeView, CScrollView)

BEGIN_MESSAGE_MAP(CLifeView, CScrollView)
	//{{AFX_MSG_MAP(CLifeView)
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_TIMER()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CLifeView construction/destruction

CLifeView::CLifeView()
{
	pDoc=NULL;	//    
}

CLifeView::~CLifeView()
{
}

BOOL CLifeView::PreCreateWindow(CREATESTRUCT& cs)
{
	return CScrollView::PreCreateWindow(cs);
}

/////////////////////////////////////////////////////////////////////////////
// CLifeView drawing

void CLifeView::OnDraw(CDC* pDC)
{
	CLifeDoc* pDoc = GetDocument();
	ASSERT_VALID(pDoc);
					//       
	pDC->SetMapMode(MM_TEXT);
	CPen *pP = pDC->SelectObject (&pen);
					//    
	CBrush *pB = pDC->SelectObject (&NewBr);
	DrawCells (pDC, Young);
					//  
	pDC->SelectObject (&OldBr);
	DrawCells (pDC, Old);
					//    
	pDC->SelectObject (&DiedBr);
	DrawCells (pDC,Died);
					//     
	CBrush clear (GetSysColor (COLOR_WINDOW));
	pDC->SelectObject (&clear);
	DrawCells (pDC,Erase);
					//   GDI
	pDC->SelectObject (pP);
	pDC->SelectObject (pB);
}

void CLifeView::OnInitialUpdate()
{
	CScrollView::OnInitialUpdate();
	pDoc = GetDocument();
	InitView();	
	pDoc->SetTitle ("Enter Life Cells with the mouse clicks and Press Go");
	pDoc->OnViewControl();
}

/////////////////////////////////////////////////////////////////////////////
// CLifeView diagnostics

#ifdef _DEBUG
void CLifeView::AssertValid() const
{
	CScrollView::AssertValid();
}

void CLifeView::Dump(CDumpContext& dc) const
{
	CScrollView::Dump(dc);
}

CLifeDoc* CLifeView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CLifeDoc)));
	return (CLifeDoc*)m_pDocument;
}
#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CLifeView message handlers

void CLifeView::OnLButtonDown(UINT nFlags, CPoint point) 
{
	if (!(pDoc->bEditMode))
		return;			//    
						//     (   )
	SetCapture();
	GrabPoint (point);	//    
}

void CLifeView::OnLButtonUp(UINT nFlags, CPoint point) 
{
	if (!pDoc->bEditMode || GetCapture() != this)
		return;					//    
	ReleaseCapture();			//  
}

void CLifeView::OnMouseMove(UINT nFlags, CPoint point) 
{
	if (GetCapture() != this)
		return;
	GrabPoint(point);			//   
}

void CLifeView::OnRButtonDown(UINT nFlags, CPoint point) 
{								//   
	pDoc->bEditMode = false;
	if (pDoc->pCtrlDlg)
		pDoc->pCtrlDlg->EnableButtons(false);
								//  
	SetTimer (pDoc->uTimerId, pDoc->uTime, NULL);
}


void CLifeView::OnTimer(UINT nIDEvent) 
{
	pDoc->NextStage();		//     	
	CScrollView::OnTimer(nIDEvent);
}

void CLifeView::OnUpdate(CView* pSender, LPARAM lHint, CObject* pHint) 
{
	Invalidate(FALSE);	
}

void CLifeView::DrawCells (CDC *pDC, LifeStatus s)
{
	UINT c = pDoc->uScale, y, x;
	for (y=1; y<pDoc->uY; y++)
		for (x=1; x<pDoc->uX; x++)
			if ( (LifeStatus)pDoc->fNew[y][x] == s )
				pDC->Ellipse (x*c,y*c,(x+1)*c,(y+1)*c);
}

void CLifeView::InitView()
{
	NewBr.DeleteObject();
	OldBr.DeleteObject();
	DiedBr.DeleteObject();
							//      
	NewBr.CreateSolidBrush (pDoc->NewColor);
	OldBr.CreateSolidBrush (pDoc->OldColor);
	DiedBr.CreateSolidBrush (pDoc->DiedColor);
	pen.DeleteObject();		//     
	pen.CreatePen (PS_SOLID,1,GetSysColor(COLOR_WINDOW));
	ChangeScroll();			//   
	Invalidate();
}

void CLifeView::ChangeScroll()
{
	UINT s = pDoc->uScale;	//   (  )
											//   
	CSize szTotal = CSize ((pDoc->uX+1)*s, (pDoc->uY+1)*s);
	CSize szPage  = CSize (5*s, 5*s);		//  
	CSize szLine  = CSize (s, s);			//  
											//   
	SetScrollSizes (MM_TEXT, szTotal, szPage, szLine);
	CPoint is;						//    
	if (pDoc->bEditMode)
		is = CPoint(1,1);		//   
	else	
		is = pDoc->ptScroll;	// ,   
	is.x *= s;
	is.y *= s;						//  
	ScrollToPosition (is);	//     
	CString t;
	t.Format("  Scale = %u ",s);		// 
	pDoc->SetTitle (t);					//    
}

void CLifeView::GrabPoint (CPoint point)
{
						//    
	CPoint p = GetScrollPosition();
						//    
	UINT y = (p.y+point.y)/pDoc->uScale,
			x = (p.x+point.x)/pDoc->uScale;
	if (y < pDoc->uY && x < pDoc->uX)
	{					//     
						//      
		pDoc->fNew[y][x] = Young;
		Invalidate();	//   
	}
}
